@@ -6,6 +6,7 @@ Created on 2023-06-11
 import json
 import os
 from dataclasses import dataclass, field
+from datetime import datetime
 from json.decoder import JSONDecodeError
 from typing import Any, Dict, List, Optional, Tuple, Union
 
@@ -28,12 +29,13 @@ class RingSpec:
         text_mode (Optional[str]): The mode of text display on the ring. Default is None.
         inner_ratio (Optional[float]): The inner radius of the ring, relative to the chart size.
         outer_ratio (Optional[float]): The outer radius of the ring, relative to the chart size.
+        symmetry_mode (Optional[str]): Specifies the symmetry mode for the ring. Supports "count", "time", "score" to determine how the ring segments are balanced. Default is None.
     """
-
     text_mode: Optional[str] = "empty"
     inner_ratio: Optional[float] = None
     outer_ratio: Optional[float] = None
     levels_visible: Optional[bool] = False
+    symmetry_mode: Optional[str] = None  
 
     @property
     def empty(self) -> bool:
@@ -68,6 +70,12 @@ class CompetenceElement:
     description: Optional[str] = None
     color_code: Optional[str] = None
     border_color: Optional[str] = None
+    
+    time: Optional[float] = None
+    time_unit: Optional[str] = None # h/CP
+    max_score: Optional[float] = None # 100
+    score_unit: Optional[str] = None # %
+    
 
     def __post_init__(self):
         # Set the id to the the slug of the name if id is None
@@ -125,11 +133,7 @@ class CompetenceFacet(CompetenceElement):
 
     This class can include additional properties or methods specific to a competence facet.
     """
-
-    time: Optional[float] = None
-    time_unit: Optional[str] = "h"
-    max_score: Optional[float] = 100.0
-    score_unit: Optional[str] = "%"
+    
 
 
 @dataclass_json
@@ -156,7 +160,6 @@ class CompetenceAspect(CompetenceElement):
     """
 
     areas: List[CompetenceArea] = field(default_factory=list)
-    credits: Optional[int] = None
 
 
 @dataclass_json
@@ -207,7 +210,6 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
         initalize the path variables of my hierarchy
         """
         super().__post_init__()
-        self.total_elements = {"aspects": 0, "areas": 0, "facets": 0}
         self.update_paths()
         self.calculate_ring_specs("empty")
 
@@ -232,11 +234,73 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
                     inner_ratio=inner_ratio,
                     outer_ratio=outer_ratio,
                 )
+                
+    def set_symmetry_mode(self,symmetry_level:str,symmetry_mode:str):
+        """
+         Sets a new symmetry mode for a specified level 
+         in the competence tree. It resets the symmetry 
+         mode for all levels to None and then 
+         applies the new symmetry mode to the 
+         specified level. This method ensures that 
+         only one level has a symmetry mode set at any given time.
+
+        Args:
+            symmetry_level(str): The level to apply the new symmetry mode. Valid levels include "tree", "aspect", "area", and "facet".
+            symmetry_mode (str): The symmetry mode to set for the specified level. Valid modes are "count", "score", and "time".
+        """
+        # reset all ring specs
+        for rl in ["tree", "aspect", "area", "facet"]:
+            self.ring_specs[rl].symmetry_mode=None
+        if symmetry_level in self.ring_specs:
+            self.ring_specs[symmetry_level].symmetry_mode=symmetry_mode
+                
+    def get_symmetry_spec(self) -> Tuple[str, str]:
+        """
+        Get the symmetry specification from the ring_specs, ensuring only one spec has the symmetry mode set.
+
+        Returns:
+            Tuple[str, str]: A pair of symmetry mode and the level it is applied to.
+
+        Raises:
+            ValueError: If multiple RingSpecs have a symmetry mode set.
+        """
+        
+        symmetry_modes = [(level, spec.symmetry_mode) for level, spec in self.ring_specs.items() if spec.symmetry_mode]
+        if len(symmetry_modes) > 1:
+            raise ValueError("Symmetry mode set for multiple ring specs.")
+        elif len(symmetry_modes) == 1:
+            level,mode=symmetry_modes[0]
+        else:
+            # Default to "count" and CompetenceFacet level if no spec is given
+            level,mode= ("facet","count")
+        if not mode in ["count","score","time"]:
+            raise ValueError(f"Invalid symmetry mode {mode} - must be count, score or time")
+        return (level,mode)
+    
+    def get_elements_for_level(self,level:int)->List[CompetenceElement]:
+        """
+        get the elements for the given hierarchy level
+        
+        Args:
+            level(int): the hierarchy level
+            
+        Returns:
+            List(CompetencElement): the list of elements on this level
+        """
+        level_name=self.level_names[level]
+        elements=self.elements_by_level[level_name]
+        return elements
+
 
     def update_paths(self):
         """
         update my paths
         """
+        self.level_names = ["tree","aspect", "area", "facet"]
+        self.level_attr_names = [None,"aspects", "areas", "facets"]
+        self.total_elements = {"tree": 1, "aspects": 0, "areas": 0, "facets": 0}
+        self.elements_by_level = {"tree": [self],"aspect": [], "area": [], "facet": []}  # Reset for re-calculation
+ 
         self.path = self.id
         self.total_levels = 1
         self.elements_by_path = {self.path: self}
@@ -247,6 +311,7 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
             self.elements_by_path[aspect.path] = aspect
             self.total_elements["aspects"] = self.total_elements["aspects"] + 1
             self.total_levels = 2
+            self.elements_by_level["aspect"].append(aspect)
             for area in aspect.areas:
                 self.total_levels = 3
                 area.competence_tree = self
@@ -254,6 +319,7 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
                 area.path = f"{self.id}/{aspect.id}/{area.id}"
                 self.elements_by_path[area.path] = area
                 self.total_elements["areas"] = self.total_elements["areas"] + 1
+                self.elements_by_level["area"].append(area)
                 for facet in area.facets:
                     self.total_levels = 4
                     facet.competence_tree = self
@@ -261,6 +327,7 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
                     facet.path = f"{self.id}/{aspect.id}/{area.id}/{facet.id}"
                     self.elements_by_path[facet.path] = facet
                     self.total_elements["facets"] = self.total_elements["facets"] + 1
+                    self.elements_by_level["facet"].append(facet)
 
     @classmethod
     def required_keys(cls) -> Tuple:
@@ -392,7 +459,6 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
             box_height,
         )
 
-
 @dataclass_json
 @dataclass
 class Achievement:
@@ -475,6 +541,27 @@ class Learner:
         )
         return file_name
 
+    @property
+    def most_recent_achievement_iso_date(self) -> Optional[str]:
+        """
+        Get the most recent achievement date in ISO format.
+
+        Returns:
+            Optional[str]: The ISO date string of the most recent achievement, or None if there are no achievements.
+        """
+        if not self.achievements:
+            return None
+        dates = [
+            achievement.date_assessed_iso
+            for achievement in self.achievements
+            if achievement.date_assessed_iso
+        ]
+        if not dates:
+            return None
+        # Parse the ISO dates and return the most recent one
+        most_recent_date = max(datetime.fromisoformat(date) for date in dates)
+        return most_recent_date.isoformat()
+
     def get_achievement_index(self, path) -> int:
         a_index = self.achievement_indices_by_path.get(path)
         return a_index
